
/** @file test_Mk5B is test reading Mk5B data 
 *  @author M.Sekido
 * @Version $Rev: 161 $
 * @Date  $Date: 2015-08-20 15:20:59 +0900 (木, 20  8月 2015) $
 * $Id: m5b_chk2.cpp 161 2015-08-20 06:20:59Z vlbi $ 
 */

#include <iostream>
#include <cstdio>
#include <cstdlib>
#include <vector>
#include <string>
#include "Mk5B_Frame.hpp"
#include "VsiData.hpp"
#include "hist_g.hpp"
using namespace std;
#include <boost/algorithm/string.hpp>
#include "conf_utl.hpp"

#include "VirDumper.hpp"

extern "C"{
#include <unistd.h>
}

/**! "main_cmn.hpp": Common Declraration of static parameters. 
 * Should be included in main file only. */
#include "main_cmn.hpp"

/** General Configulation data */
Config_set vdif_conf;


void usage(){
  cerr << "Usage: " << me << " -f <filename>  [-d <bit:nch>] \n";
  cerr << "        [-b <bit encode>] [-n <int> | -t <sec>] \n";
  //  cerr << " -m : Mode information, nch,bit, Sampling rate (MHz) \n";
  cerr << " -n : Every frame Display Mode. \n";
  cerr << "      Number of frames to be displayed. (default:10)\n";
  cerr << " -t : Every Sec. Display Mode.\n";
  cerr << "      Duration to check the data. Default=10sec. 0= whole data.\n";
  cerr << " -d : Specify the sampling mode.\n";
  cerr << " -b : Choice of bit encoding for histgram.\n";
  cerr << "      one of [ADS2000/ADS3000/SignMag/RevSign].(default=ADS2000)\n";
  cerr << "           ADS2000: 00,01,10,11 from bottom to top level.\n";
  cerr << "           ADS3000: 00,10,01,11 from bottom to top level.\n";
  cerr << "           SignMag: 01,00,10,11 from bottom to top level.\n";
  cerr << "           RevSgn : 10,00,01,11 from bottom to top level.\n";
  exit(1);
}

int main(int argc, char *argv[]){
  me = (me = strrchr(argv[0], '/')) == NULL ? argv[0] : me+1; /* My name */
  int k=0;
  string fname;
  Mk5B_Frame m5b_frame;
  int nch(1), qbit(1);
  uint32_t srate(16000000);
  VsiData::BitEnc_t code_type = VsiData::ADS2000_2bit;
  int ndisp =10;
  bool every_frame = false;
  // int dur=10;

  while (k >= 0) { 
    switch (k = getopt(argc, argv, "hf:b:d:n:")) { 
    case 'h':  usage(); break;
    case 'f': fname = optarg; break;
    case 'n': ndisp = atoi(optarg); every_frame = true;  break;
      //    case 't': dur = atoi(optarg);  ndisp=0;    break;
    case 'd': {
      vector <string> v;
      boost::algorithm::split(v,optarg,boost::is_any_of(":"));
      if(v.size() < 2){cerr << "E: -m option failed\n"; usage();}
      qbit =  atoi(v[0].c_str());
      nch  =  atoi(v[1].c_str());
      m5b_frame.Set_Sparam(nch,qbit,srate);
      every_frame = true;
      break;
    }
    case 'b':
      if( strncmp(optarg, "ADS3000",7)==0) code_type = VsiData::ADS3000_2bit;
      if( strncmp(optarg, "SignMag",7)==0) code_type = VsiData::SignMag_2bit;
      if( strncmp(optarg, "RevSign",7)==0) code_type = VsiData::RevSgnMag_2bit;
      break;
      //    case 'd': mk5b_hdr.Set_dbg(atoi(optarg)); break;
    case '?' : /* Error:  character not in list */ 
      fprintf(stderr, /* Yes */ 
	      "%s WARNING:  Unknown option on command line \n", me); 
      break; 
    case ':' : /* Error:  missing parameter */ 
      fprintf(stderr, "%s WARNING:  Missing number on command line \n", me); 
      usage();
      break; 
    }
  }

  if(fname.empty()) usage();
  if(! m5b_frame.ROpen(fname)){
    cerr << "Error inn opening file: " << fname << endl;
    exit(1);
  }
  

  m5b_frame.Fix_nFrame_sec();

  vector <Mk5B_Hdr> hdrarr;
  uint32_t nframe;

  if( every_frame)  nframe = m5b_frame.Read_every_frame_hdr(hdrarr,ndisp);
  else              nframe = m5b_frame.Read_every_sec_hdr(hdrarr, ndisp);
  
  if(nframe < 1){
    cerr << "E Failed to find Mk5B Header.\n";
    exit(1);
  }
  
  long datsize = m5b_frame.Framesize() - m5b_frame.Headsize();
  int npt_frame = datsize * 8/qbit/nch; // #point per frame;
  int lbound(0), ndiv(2), istep(0);
  switch(qbit){
  case 1: lbound = 0, istep=2; ndiv=2; break;
  case 2: lbound =-2; istep=2; ndiv=4;  break;
  case 4: lbound =-7, istep=1; ndiv=16; break;
  case 8: lbound =-127, istep=1; ndiv=256; break;
  }
  //  Hist_g <int8_t> hist( lbound, ndiv,istep);
  Hist_g <float> hist( lbound, ndiv,istep);
  
  cout << "========= M5B_FRAME check  ================\n";
  double dsrate = (double)nframe * datsize * 8/hdrarr.size()/qbit/nch*1.e-6;
  fprintf(stdout," Reading Nsec =%ld sec, %lu frames, Rate(Estimate) = %.3e Msps\n", 
	  hdrarr.size()-1, (unsigned long)nframe, dsrate); 
  if( every_frame){
    if(! m5b_frame.Rewind()) {cerr << "E Faile to m5b_frame.Rewind()\n"; exit(1);}
    vector <uint8_t> rbuf;
    DAS_Info das = m5b_frame.DAS_Inf();// Sampling info
    VsiData vsidat(&das);
    vsidat.Make_Table(code_type); // Make Decode Table 
    for(int i =0; i < (int)ndisp; i++){
      if(m5b_frame.Read_frame(hdrarr, rbuf, 1) < 1){
	fprintf(stdout, "E Failed to read frame\n"); break;
      }
      // Histgram Compute 
      vsidat.Interpret(rbuf,npt_frame);
      fprintf(stdout, "%s ", hdrarr[0].Str_hd().c_str());
      if(nch > 7)fputc('\n',stdout);
      fprintf(stdout,"Hist[%%]");
      for(int ich =0; ich<nch; ich++){
	hist.Init();
	hist.Accm(vsidat.fValArr(ich));
	string str = hist.StrRatioArr(); 
	fprintf(stdout,"(%dch):%s ", ich, str.c_str());
      }
      fputc('\n', stdout);
    }
  }else{
    for(int i=0; i < (int)hdrarr.size(); i++){
	hdrarr[i].Print_hd(stdout);
	//fprintf(stdout," diff = %d\n", hdrarr[i].iFrame() - hdrarr[i-1].iFrame());
	//      if(ndisp !=0 && i > (int)ndisp) break;
    }

  }
  cout << "==========================================\n";
  
}

