#ifndef __LOGSET_HPP___
/** 
 * @Version $Rev: 56 $
 * @Author M.Sekido
 * @Date  $Date: 2013-07-03 20:40:14 +0900 (水, 03  7月 2013) $
 * $Id: LogSet.hpp 56 2013-07-03 11:40:14Z vlbi $ 
 */

#include "Logsystem.hpp"

class LogSet{
public:
  LogSet(const char* prog, const int verb=0);
  void Set_errfile(const char* f){err_file=f;}
  void Set_infofile(const char* f){info_file=f;}
  void Set_warnfile(const char* f){warn_file=f;}
  void Set_debgfile(const char* f){debug_file=f;}
  void Set_level(const int v){verbose_level=v;}
  void Start();
  void Stop();
private:
  string err_file;
  string debug_file;
  string info_file;
  string warn_file;
  int verbose_level;
};


inline LogSet::LogSet(const char* prog, const int verb):verbose_level(verb){
  string logdir = "/tmp/";
  debug_file = logdir + prog + "_dbg.txt";
  info_file = logdir + prog + "_info.txt";
  warn_file = logdir + prog + "_warn.txt";
  err_file = logdir + prog + "_err.txt";
}

void LogSet::Start(){

  if(Logsystem::debug_logger()->set_stream(debug_file) <0)
    fprintf(stderr, "E LogSet::Start(): faile to open debug_file(%s) \n", debug_file.c_str());
  if(Logsystem::info_logger()->set_stream(info_file) < 0)
    fprintf(stderr, "E LogSet::Start(): faile to open info_file(%s) \n", info_file.c_str());
  if(  Logsystem::warning_logger()->set_stream(warn_file) < 0)
    fprintf(stderr, "E LogSet::Start(): faile to open warn_file(%s) \n", warn_file.c_str());
  if(Logsystem::error_logger()->set_stream(err_file) < 0)
    fprintf(stderr, "E LogSet::Start(): faile to open err_file(%s) \n", err_file.c_str());
   
  switch(verbose_level){
    Logger::PrntLevel_t t;
  case 0:
    t = Logger::MINIMUM;
    Logsystem::warning_logger()->enable_logging(t); 
    Logsystem::info_logger()->enable_logging(t);
    Logsystem::error_logger()->enable_logging(t);
    break;
  case 1:
    t = Logger::SMALL;
    Logsystem::debug_logger()->enable_logging(t); 
    Logsystem::info_logger()->enable_logging(t);
    Logsystem::error_logger()->enable_logging(t);
    Logsystem::warning_logger()->enable_logging(t);
    break;
  case 2:
    t = Logger::MEDIUM;
    Logsystem::debug_logger()->enable_logging(t); 
    Logsystem::info_logger()->enable_logging(t);
    Logsystem::error_logger()->enable_logging(t);
    Logsystem::warning_logger()->enable_logging(t);
    break;
  case 3:
    t = Logger::DETAIL;
    Logsystem::debug_logger()->enable_logging(t); 
    Logsystem::info_logger()->enable_logging(t);
    Logsystem::error_logger()->enable_logging(t);
    Logsystem::warning_logger()->enable_logging(t);
    break;
  case 4:
    t = Logger::ALL;
    Logsystem::debug_logger()->enable_logging(t); 
    Logsystem::info_logger()->enable_logging(t);
    Logsystem::error_logger()->enable_logging(t);
    Logsystem::warning_logger()->enable_logging(t);
    break;
  default:
    cerr << " No Logging.\n";
    break;
  }
  Logsystem::debug_logger()->Tstamp() << "==== Logging start =====\n";
  Logsystem::info_logger()->Tstamp() << "==== Logging start =====\n";
  Logsystem::error_logger()->Tstamp() << "==== Logging start =====\n";
  Logsystem::warning_logger()->Tstamp() << "==== Logging start =====\n";

}

inline void LogSet::Stop(){
  Logsystem::debug_logger()->close_stream();
  Logsystem::info_logger()->close_stream();
  Logsystem::error_logger()->close_stream();
  Logsystem::warning_logger()->close_stream();
}


#endif
