/** 
 * @Version $Rev: 56 $
 * @Author M.Sekido
 * @Date  $Date: 2013-07-03 20:40:14 +0900 (水, 03  7月 2013) $
 * $Id: Rate_meas.cpp 56 2013-07-03 11:40:14Z vlbi $ 
 */

#include "Rate_meas.hpp"
#include <algorithm>
using namespace std;

/// Constructor 
Rate_Meas::Rate_Meas():
  rcv_Mbps(0),snd_Mbps(0),rcv_total(0),snd_total(0),
  rcv_pld_total(0),snd_pld_total(0),
  run_state(false),sock_(0),update_intvl(DEFAULT_UPDATETIME),
  disp_sw(Rate_Meas::OFF),target_rate_Mbps(0), workmode(TX),
  pktRegTbl(DEFAULT_NPKT),cntMis(0),seq0(-1)
{ 
  thread_number++;
  pktCnt[0] =pktCnt[1] = 0;
  cntDup[0] = cntDup[1] = 0;
  iseq_max[0] = iseq_max[1] = 0;
  ReInit();
  char buf[256];
  sprintf(buf,"RM%d", thread_number);
  SetID(buf); // Set Thread ID
}


void *Rate_Meas::thread_func(void){
  gettimeofday(&st,NULL);  
  st0 = st;
  run_state=true;
  Measurement();
  return NULL;
} 


void Rate_Meas::stop(){
  //  usleep(500000);
  if(run_state) {
    run_state=false; 
    //    if(join() ==0){
    //    Thread::stop();
    Logsystem::debug_logger()->Tstamp()
      << "Rate_Meas[" << Thread::StrID()
      << "]::Measuremet() stop-joined.\n";
    Logsystem::debug_logger()->Flush();
    cerr << "Rate_Meas[" << Thread::StrID() 
	 << "]::Measuremet() stop-joined." << endl;
    //    }
  }
  ReInit();
}


string Rate_Meas::Epoch()const{
  struct tm tm;
  string t;
  if(gmtime_r(&et.tv_sec,&tm) ==NULL){
    cerr << "Error in Rate_Meas::Epoch():Failed gettime_r()\n";
    return t;
  }
  char buf[80];
  strftime(buf, 80,"%Y/%j %H:%M:%S", &tm);
  t = buf;
  return t;
}


void Rate_Meas::ReInit(const int flag){
  if(flag <1){
    rcv_Mbps = snd_Mbps = 0.0;
    pld_rcv_Mbps = pld_snd_Mbps = 0.0;
    rcv_total = snd_total = rcv_total_st0 = snd_total_st0 = 0;
    rcv_pld_total = snd_pld_total = rcv_pld_total_st0 = snd_pld_total_st0 = 0;
    recomended_wait = 0.0;
    seq0 = -1; 
  }
  cntDup[0] = 0;
  pktCnt[0] = 0;
  iseq_max[0] = 0;
  fill(pktRegTbl.begin(),pktRegTbl.end(),false); 
}


void Rate_Meas::TakeStatistics(){
  //  cerr << "Computing Statistics" << endl;
  _mutex.lock();
  cntMis = count(pktRegTbl.begin(),pktRegTbl.begin() + iseq_max[0], false);
  cntDup[1] = cntDup[0]; // Rsearve last counter value;
  if(pktCnt[0] != 0 && 
     pktCnt[0] != iseq_max[0]+1){ // Because iseq start from 0;
    Logsystem::debug_logger()->Tstamp()
      << "Warning pktCnt(" << pktCnt[0] << ")!= iseq_max("
      << iseq_max[0] << ")\n";
    Logsystem::debug_logger()->Flush();
    
  }
  pktCnt[1] = pktCnt[0]; // Store the counter data to register 
  iseq_max[1] = iseq_max[0];

  ReInit(1);
  seq0=-1;
  _mutex.unlock();

}

