/** Conf Utility C++ Libraly
 * coded by 
 * @Author M.Sekido
 * @Version $Rev: 56 $
 * @Date  $Date: 2013-07-03 20:40:14 +0900 (水, 03  7月 2013) $
 * $Id: conf_utl.hpp 56 2013-07-03 11:40:14Z vlbi $ 
 */

#ifndef __CONF_UTLTY__
#define __CONF_UTLTY__


#include <fstream>
#include <boost/algorithm/string.hpp>
#include <vector>
#include <string>
#include <iostream>
#include <map>

const std::string DEFAULT_CONF_FILE="/usr/local/etc/vdif.conf";


class Config_set {
  std::map <std::string, std::string> list;
public:
  Config_set(){;}
  const int Read(const char* fname);
  std::string Value(const std::string& key) const;
  void Print(FILE *fp=stdout);
  const ulong Nentry()const {return list.size();}
  const bool Check_Key(const std::string& key)const {return (list.find(key) != list.end()); };
};
  

inline void Config_set::Print(FILE *fp){
  std::map<std::string, std::string>::iterator itr = list.begin();
  int cnt=0;
  while(itr != list.end()){
    fprintf(fp,"[%2d] %s: %s \n", cnt++, itr->first.c_str(), itr->second.c_str());
    itr++;
  }
  
}

inline std::string Config_set::Value(const std::string& key) const{
  if(list.find(key) == list.end()){
    return "NODATA";
  } else
    //    return list.find(key)->second;
    return list.find(key)->second;
}

inline  const int Config_set::Read(const char*fname){
  std::ifstream fin(fname); 
  const int MAXLINE =1024;
  char buf[MAXLINE];
  
  if(!fin){
    fprintf(stderr, "E Failed to open Config file[%s] \n", fname);
    return -1;
  }
  
  std::vector <std::string> v;
  while(!fin.eof()){
    fin.getline(buf,MAXLINE);
    if(buf[0]=='#')continue;
    std::string s1(buf);
    boost::algorithm::split(v,s1 , boost::is_any_of(" :;"), boost::token_compress_on);
    if(v.size() < 2){
      v.resize(0);
    }else{
      printf("s1= %s list[%s] = [%s] \n", s1.c_str(), v[0].c_str(), v[1].c_str());
      list[v[0]] = v[1];
    }
    
  } // end of read

  return 0;
  
}

#endif
